#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    pid_t ttyPID = 0;
    int masterin = 0;
    char slavename[] = "TTY for Dirks Emulator";
    ui->setupUi(this);

    m_pid = 0;
    envlq = QProcessEnvironment::systemEnvironment();

    m_worker = new WorkerThread(this);

    setFixedSize(720, 600);

    m_slaveFD = 0;

    /*
        m_shell = new QProcess();
        m_shell->setProcessChannelMode(QProcess::MergedChannels);

        QString shellName = "/bin/bash";
        QStringList arguments;
        arguments << "-i" ;
        m_shell->start(shellName, arguments);
        m_shell->waitForStarted();

        m_pid = m_shell->processId();   */

    ttyPID = frkpty(&masterin, slavename);

    if (ttyPID > 0) {
        m_pid = ttyPID;
        m_worker->setTTY(ttyPID);
        m_worker->setHandle(masterin);

        m_worker->m_pollOutput.fd = masterin;
        m_worker->m_pollOutput.events = POLLIN;

        connect(m_worker, &WorkerThread::child_sent, ui->textEdit, &ExtEdit::receiveText);
        connect(ui->textEdit, &ExtEdit::forwardText, this, &MainWindow::child_receive);

        connect(ui->textEdit, &ExtEdit::askSIGINT, this, &MainWindow::doSIGINT);
        connect(ui->textEdit, &ExtEdit::askSIGSTP, this, &MainWindow::doSIGSTP);

        m_worker->start();
        ui->textEdit->setFocus();
    }
}

MainWindow::~MainWindow()
{
    m_worker->terminate();
    m_worker->wait();
    delete m_worker;
    delete ui;
}

/*
void MainWindow::child_send_standard() {
    QByteArray data = m_shell->readAllStandardOutput();
    emit child_sent(QString(data));
}

void MainWindow::child_send_error() {
    QByteArray data = m_shell->readAllStandardError();
    emit child_sent(QString(data));
}
    */

void MainWindow::child_receive(QString stream)
{
    int length = 0;
    length = stream.length();
    ::write(m_worker->m_fDes, (char *) stream.toStdString().c_str(), length);
}

void MainWindow::doSIGINT()
{
    QString command = QString("/usr/bin/pkill -2 -P ") + QString::number(m_pid);
    system((char *) command.toStdString().c_str());
}

void MainWindow::doSIGSTP()
{
    QString command = QString("/usr/bin/pkill -20 -P ") + QString::number(m_pid);
    system((char *) command.toStdString().c_str());
}

pid_t MainWindow::frkpty(int *masterin, char *slavename)
{
    int master, slave;
    pid_t chldp;

    char *envv[4];
    char env1[] = "TERM=VT220";

    QString displayString = QString("DISPLAY=") + envlq.value(QString("DISPLAY"));
    QString homeString = QString("HOME=") + envlq.value(QString("HOME"));

    envv[0] = env1;
    envv[1] = (char *) displayString.toStdString().c_str();
    envv[2] = (char *) homeString.toStdString().c_str();
    envv[3] = NULL;

    if ( ::openpty(&master, &slave, slavename, NULL, NULL) < 0) {
        FAIL("Could not create pseudo-tty.");
        exit(1);
    }

    chldp=fork();
    if (chldp == -1) {
        FAIL("Could not fork.");
        exit(1);
    } else if (chldp == 0) {
        ASSERT_IS(0, ::close(master));

        if (ioctl(slave, TIOCSCTTY) < 0) {
            /*            FAIL("Inapprpriate I/O control detected.");
                        exit(1);    */
        }

        ASSERT_NOT(-1, setsid());

        ASSERT_NOT(-1, dup2(slave, STDIN_FILENO));
        ASSERT_NOT(-1, dup2(slave, STDOUT_FILENO));
        ASSERT_NOT(-1, dup2(slave, STDERR_FILENO));

        execle("/bin/bash", "bash", "-i", NULL, envv);

        return 0;
    } else {
        *masterin=master;

        m_slaveFD = slave;
        QTimer::singleShot(500, this, &MainWindow::disown);

//        ASSERT_IS(0, ::close(slave));
        return chldp;
    }
    exit(1);
}

void MainWindow::disown()
{
    ::close(m_slaveFD);
}

